/* -*- c -*- 
 * File: iterator.h
 * Author: Igor Vlasenko <vlasenko@imath.kiev.ua>
 * Created: Mon Jul 30 20:06:20 2007

 * TODO: verify that tb_iter use checks on new() only

 * smart iterators with default value support
 *
 */


class num_iter {
  const table<noid_type, noid_type> *tbl;
  size_t cur;
  size_t border;
  bool is_emulate;
  large_int emulated_key;
  large_int emulated_val;
  NodeConst::TYPE keytype;
  NodeConst::TYPE valtype;
  char* top;
public:
  num_iter(const table<noid_type, noid_type>* ctbl, size_t ccur=0) : tbl(ctbl), cur(ccur) {
    border=ctbl->get_size();
    keytype=tbl->get_keytype();
    valtype=tbl->get_valtype();
    top = tbl->get_data_offset();
    is_emulate = false;
    if (ccur<0 || ccur>border) throw sdb_error("num_iter: bad index");
  }

  num_iter(large_int lemulated_key, large_int lemulated_val, size_t ccur=0) {
    cur=ccur;
    border=1;
    is_emulate=true;
    emulated_key=lemulated_key;
    emulated_val=lemulated_val;
    tbl=NULL;
    if (ccur<0 || ccur>border) throw sdb_error("num_iter: bad index");
  }

  friend bool operator==(const num_iter& a, const num_iter& b) {
    return a.cur==b.cur;
  }

  friend bool operator!=(const num_iter& a, const num_iter& b) {
    return a.cur!=b.cur;
  }

  friend size_t operator-(const num_iter& a, const num_iter& b) {
    return a.cur-b.cur;
  }
  num_iter& operator++(void);
  num_iter& operator--(void);

  bool is_end(void) const { return cur==border; }

  large_int get_key(void) const;
  large_int get_val(void) const;

  static std::pair<num_iter, num_iter> map_numeric_key(table < noid_type,noid_type > *tbl, large_int n);
  static std::pair<num_iter, num_iter> map_string_key(table < noid_type,noid_type > *tbl, char* str, NodeConst::comparison_type type);

  inline static std::pair<num_iter, num_iter> defvalue_pair(large_int lemulated_key, large_int lemulated_val) {
    return std::pair<num_iter,num_iter>(num_iter(lemulated_key,lemulated_val,0),num_iter(lemulated_key,lemulated_val,1));
  }
  inline static std::pair<num_iter, num_iter> empty_pair(void) {
    return std::pair<num_iter,num_iter>(num_iter(0,0,0),num_iter(0,0,0));
  }
};

inline num_iter& num_iter::operator++(void) {
  if (cur < border) cur++;
  return *this;
}

inline  num_iter& num_iter::operator--(void) {
  if (cur>0) cur--;
  return *this;
}

large_int num_iter::get_key(void) const { 
  if (is_emulate) return emulated_key;
  if (keytype==NodeConst::INT8) return ((const int8 *)(top+tbl->get_key(cur)))->to_long_long();
  return tbl->get_key(cur);
 };

large_int num_iter::get_val(void) const { 
  if (is_emulate) return emulated_val;
  if (valtype==NodeConst::INT8) {
    return ((const int8 *)(top+tbl->get_val(cur)))->to_long_long();
  }
  return tbl->get_val(cur);
 };

std::pair<num_iter, num_iter> num_iter::map_numeric_key(table < noid_type,noid_type > *tbl, large_int n) {
  if (tbl->get_keytype()==NodeConst::INT8) {
    std::pair<ll_tb_iter, ll_tb_iter> p = tbl->map_int8_key(int8(n));
    if (p.first.is_end() && tbl->has_defVal()) {
      large_int defval = tbl->get_defVal();
      defval=((const int8 *)(tbl->get_data_offset()+defval))->to_long_long();
      return defvalue_pair(n,defval);
    } else {
      return std::pair<num_iter,num_iter>(num_iter(tbl, p.first.get_cur()), num_iter(tbl, p.second.get_cur()));
    }
  } else {
    std::pair<ll_tb_iter, ll_tb_iter> p= tbl->map_numeric_key(n);
    if (p.first.is_end() && tbl->has_defVal()) {
      large_int defval = tbl->get_defVal();
      return defvalue_pair(n,defval);
    } else {
      return std::pair<num_iter,num_iter>(num_iter(tbl, p.first.get_cur()), num_iter(tbl, p.second.get_cur()));
    }
  }
}

std::pair<num_iter, num_iter> num_iter::map_string_key(table < noid_type,noid_type > *tbl, char* str, NodeConst::comparison_type type) {
  if (tbl->get_keytype()!=NodeConst::STRING4) {
    // sdb_error: complain
    return empty_pair();
  } else {
    std::pair<ll_tb_iter, ll_tb_iter> p= tbl->map_string_key(str, type);
    if (p.first.is_end() && tbl->has_defVal()) {
      large_int defval = tbl->get_defVal();
      return defvalue_pair(str - tbl->get_data_offset(), defval);
    } else {
      return std::pair<num_iter,num_iter>(num_iter(tbl, p.first.get_cur()), num_iter(tbl, p.second.get_cur()));
    }
  }
}


/*
 *  Local Variables:
 *  mode: c++
 *  End:
 */
