#ifndef __DB_HEADER_H__
#define __DB_HEADER_H__

#include <string>
#include <utility>

#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include "typedefs.h"

/*
    # header of SDB --- SDB_HEADER_LENGTH=>6 values
    seek ($IDXFILE, 0, 0);
    ######################## 4 bytes #############################
    print $IDXFILE MAGIC_STRING; # magic value 4 bytes		#0
    # ------------------------------------------------------------
    print $IDXFILE pack("L",VERSION); # version 4 bytes  	#1
    # ------------------------------------------------------------
    print $IDXFILE packoffset($INFO_OFFSET);	#4 bytes	#2
    # ------------------------------------------------------------
    print $IDXFILE packoffset($self->{ROOT_NRIF_OFFSET});	#3
    # ------------------------------------------------------------
    # end of file offset (we write new nodes there)  #4 bytes   #4
    # usually it is the end of file, but if we delete last attribute
    # without truncating the file, this offset may lay inside the file
    print $IDXFILE packoffset($self->{END_OF_FILE_OFFSET});	#5
    # ------------------------------------------------------------
 */

#define SUPPORTED_VERSION 123

class sdb_header {
  class sdb_file {
    off_t size_in_bytes;
  public:
    offset_type * top;
    sdb_file(const char* name) {
      int fd = open(name, O_RDONLY);
      std::string err = std::string("Can not open file ")+std::string(name);
      if (fd == -1) throw sdb_error(err.c_str());
      struct stat st;
      fstat(fd, &st);
      size_in_bytes = st.st_size;
      top = (offset_type *) mmap(0, size_in_bytes,  PROT_READ,  MAP_SHARED, fd, 0);
      close(fd);
    }

    ~sdb_file(void) {
      munmap((void *)top, size_in_bytes);
    }
  };
  sdb_file file;
public:
  sdb_header(const char* name) : file(name) {
    if(file.top[1] != SUPPORTED_VERSION) throw sdb_error("Usuported version ! :-(...");
  }
  NodeRIndex<noid_type,noid_type>* get_nrif(void) {
    return new NodeRIndex<noid_type,noid_type>((char *)(file.top) + file.top[3]);
  }
  const char* info(void) const {
    const char* s = (const char*) file.top;
    return s+file.top[2];
  }
};


#endif

//  Local Variables:
//  mode: c++
//  End:
