// File: search.h
//
// Created: Fri Dec 26 18:44:41 2003
//
// $Id: search.h,v 1.2 2004/04/21 11:38:09 sav Exp $
//

#ifndef __SEARCH_H__
#define __SEARCH_H__

//#include<iostream>

template<class T, class cmpT> bool lt(const T& p, const cmpT& c) {
  int cc = c(p);
  if (cc>0) return true;
  return false;
}

template<class T, class cmpT> bool lt_r(const T& p, const cmpT& c) {
  int cc = c(p);
  if (cc>0) return true;
  if (cc==0) {
    T t=p;
    ++t;
    T t1 = t;
    if(++t1==t) return false;	// t point to last element
    if(c(t) == 0) return true;
  }
  return false;
}

template<class T, class cmpT> bool gt(const T& p, const cmpT& c) {
  int cc = c(p);
  if (cc<0) return true;
  return false;
}

template<class T, class cmpT> bool gt_l(const T& p, const cmpT& c) {
  int cc = c(p);
  if (cc<0) return true;
  if (cc==0) {
    T t=p;
    if(--t==p) return false;
    if(c(t) == 0) return true;
  }
  return false;
}

template<class T, class cmpT>
class comparator {
public:
  typedef bool CMPFUNC(const T& p, const cmpT& c);
private:
  CMPFUNC *ltf;
  CMPFUNC *gtf;
  cmpT cmpf;
public:
  comparator(const cmpT& ccmpf) : cmpf(ccmpf) {}
  void set_lt(CMPFUNC *cltf) {ltf=cltf;}
  void set_gt(CMPFUNC *cgtf) {gtf=cgtf;}
  bool lt(T& p) const {
    return (*ltf)(p, cmpf);
  }
  bool gt(T& p) const {
    return (*gtf)(p, cmpf);
  }
};

template<class T, class compT> 
class finder {
  const compT& cmp;
  const T begin;
  const T end;

  T b;
  T e;
  T c;
public:
  finder(const compT &ccmp, T cbegin, T cend) : 
    cmp(ccmp), begin(cbegin), end(cend), b(cbegin), e(cend), c(cend) {}
  finder(const finder &f) : 
    cmp(f.cmp), begin(f.begin), end(f.end), b(f.b), e(f.e), c(f.c) {}
  finder & operator=(const finder &f) {
    b=f.b; e=f.e; c=f.c; return *this;
  }
  void set_left(T &r) {b=r;}
  void set_right(T &r) {e=r; ++e;}
  const T& operator()(const T& cb, const T& ce) {
    b=cb;
    e=ce;
    return find();
  }

  const T& operator()(void) { return find(); }
  
  const T& find(void) {
    // Is empty ?
    if (b==e) return end;

    // Test on range... (optional)
      --e; if(cmp.lt(e)) return end; ++e;
    if (cmp.gt(b)) return end;

    // Test binomial search ...
    while(e-b > 0) {
      c=half(b,e);
      //std::cerr << "** " << c.get_val() << " -- " << c.get_key() << "\n";
      if(cmp.lt(c)) { 
	if (b==c) return end;
	b=c;
      } 
      else if(cmp.gt(c)) e=c;
      else return c;
    }
    return end;
  }
};

template<class iterT, class cmpT>
class search {
  //  typedef bool CMPFUNC(const iterT& p, const cmpT& c);
  const iterT begin;
  const iterT end;
  iterT first;
  iterT last;
public: 
  search(const iterT& cbegin, const iterT& cend) : 
    begin(cbegin), end(cend), first(begin), last(end) { }

  bool find(const cmpT& cc, bool unique) {
    typedef  comparator<iterT, cmpT> comp_t;
    comp_t comp(cc);

    finder<iterT, comp_t> find(comp, begin, end);

    comp.set_lt(lt<iterT, cmpT>);
    comp.set_gt(gt<iterT, cmpT>);

    iterT ref = find();

    if (ref == end) return false;
    if(unique) {
      first = last = ref;
    } else {
      finder<iterT, comparator<iterT, cmpT> > find_o(find);
      find.set_right(ref);
      comp.set_gt(gt_l<iterT, cmpT>);
      first = find();
      find = find_o;
      comp.set_lt(lt_r<iterT, cmpT>);
      comp.set_gt(gt<iterT, cmpT>);
      last = find();
    }
    ++last;
    return true;
  }

  iterT get_first(void) { return first; }
  iterT get_last(void)  { return last; }
};

#endif

//  Local Variables:
//  mode: c++
//  End:
