// File: table.h
//
// Created: Mon Jan 12 16:05:20 2004
//
// $Id: table.h,v 1.4 2004/04/24 12:18:02 sav Exp $
//

#ifndef _TABLE_H_
#define _TABLE_H_

#include "sdb_error.h"
#include "stdio.h"

#include "tb_iter.h"

//#include <iostream>

template<class TD1, class TD2> 
class table {
private:
  size_t size;
  const TD1* key_ptr;
  const TD2* value_ptr;
  bool is_unique;
  bool has_def_value; 
  char* data_offset;
  TD2 defvalue;
  NodeConst::TYPE keytype;
  NodeConst::TYPE valtype;
public:

  void init (
	    NodeConst::TYPE ckeytype,
	    NodeConst::TYPE cvaltype,
	    void *start, size_t csize, 
	    void* cdata_offset=NULL,
	    bool cis_unique = false, 
	    bool chas_def_value = false, 
	    TD2 cdefvalue = 0
	     ) 
  {
    keytype = ckeytype;
    valtype = cvaltype;
    size=csize;
    key_ptr = (TD1*) start;
    value_ptr = (TD2*) (key_ptr + size);
//    std::cerr << value_ptr << " " << key_ptr  << "\n";
    is_unique = cis_unique;
    data_offset = (char*) cdata_offset;
    has_def_value = chas_def_value;
    defvalue = cdefvalue;
  }

  tb_iter<TD1> begin(void) const {
    return tb_iter<TD1>(key_ptr,size,0);
  }

  tb_iter<TD1> end(void) const {
    return tb_iter<TD1>(key_ptr,size, size);
  }

  inline TD1 get_key(size_t n) const { // NO CHECKED INDEX !!!
    return key_ptr[n];
  }

  inline TD2 get_val(size_t n) const { // NO CHECKED INDEX !!!
    return value_ptr[n];
  }

  inline NodeConst::TYPE get_keytype(void) const {return keytype;}
  inline NodeConst::TYPE get_valtype(void) const {return valtype;}
  
  std::pair<TD1, TD2> at_obj(size_t n) const { // NO CHECKED INDEX !!!
    return std::pair<TD1, TD2>(get_key(n), get_val(n));
  }

  std::pair<TD1, TD2> operator[](size_t n) const { // CHECKED INDEX !!!
    if (! check(n)) throw sdb_error("internal error: Bad index detected");
    return at_obj(n);
  }

  TD2 get_defVal(void) const {return defvalue;}
  bool has_defVal(void) const {return has_def_value;}
  char* get_data_offset(void) const {return data_offset;}

  size_t get_size(void) const {return size;}
  
  inline bool check(size_t n) const { return n>=0 && n<size;}

  std::pair< tb_iter<TD1>, tb_iter<TD1> > 
  map_numeric_key(TD1 key);
  std::pair< tb_iter<TD1>, tb_iter<TD1> > 
  map_int8_key(int8 key);
  std::pair< tb_iter<TD1>, tb_iter<TD1> > 
  map_string_key(const char* key, NodeConst::comparison_type tt = NodeConst::FULL);
};

#include<string>
#include<iostream>
#include "typedefs.h"
#include "search.h"
#include "cmps.h"

template<class TD1, class TD2>
std::pair< tb_iter<TD1>, tb_iter<TD1> > 
table<TD1, TD2>::map_numeric_key(TD1 key) {
  search< tb_iter<TD1>, num_cmp<TD1,TD2> > srch(this->begin(), this->end());
  num_cmp<TD1,TD2> lcc (key);
  if(!srch.find(lcc, this->is_unique)) 
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(this->end(), this->end()) ;
  else
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(srch.get_first(), srch.get_last());
}

template<class TD1, class TD2>
std::pair< tb_iter<TD1>, tb_iter<TD1> > 
table<TD1, TD2>::map_int8_key(int8 key) {
  search< tb_iter<TD1>, int8tmp_cmp<TD1,TD2> > srch(this->begin(), this->end());
  int8tmp_cmp<TD1,TD2> lcc (key, this->data_offset);
  if(!srch.find(lcc, this->is_unique)) 
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(this->end(), this->end()) ;
  else
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(srch.get_first(), srch.get_last());
}

template<class TD1, class TD2>
std::pair< tb_iter<TD1>, tb_iter<TD1> > 
table<TD1, TD2>::map_string_key(const char* key, NodeConst::comparison_type tt) {
  search< tb_iter<TD1>, str_cmp<TD2> > srch(this->begin(), this->end());
  str_cmp<TD2> lcc (tt, key, this->data_offset);
  if(!srch.find(lcc, this->is_unique)) 
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(this->end(), this->end()) ;
  else
    return std::pair< tb_iter<TD1>, tb_iter<TD1> >(srch.get_first(), srch.get_last());
}


#endif

//  Local Variables:
//  mode: c++
//  End:
